import random
import re
import os
import numpy as np

from typing import Tuple


class Generator:

    @staticmethod
    def do_uuid():
        """

        :return uuid.uuid1().int % (10**6) # example output = 694874
        -------

        """

        return random.randint(100000, 99999999)

    @staticmethod
    def path_url_creator(gui: bool, cfg: dict, split_data: list, i: int, directory=None) -> Tuple[str, str, str]:
        """

        :param gui:
        :param cfg:
        :param split_data:
        :param i:
        :param directory:
        :return:
        """
        host = cfg.image.ip_remote if cfg.image.Remote else cfg.image.ip_local
        if gui:
            ip_path = re.compile("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
            ip_check = bool(ip_path.match(directory))
            print("ip mi", ip_check)
            if ip_check:
                path = os.path.join(
                    "http://" + host + ":" + cfg.image.port + "/" + cfg.image.directory + ":",
                    str(split_data[i]["dirname"]), split_data[i]["filename"], split_data[i]["imgname"])
            else:
                path = os.path.join(directory, str(split_data[i]["dirname"]),
                                    split_data[i]["filename"],
                                    split_data[i]["imgname"])
        else:
            path = os.path.join(
                "http://" + host + ":" + cfg.image.port + "/" + cfg.image.directory + ":",
                str(split_data[i]["dirname"]), split_data[i]["filename"], split_data[i]["imgname"])
            outPath = os.path.join("exports")

        print('Image Path :', path)
        return host, path, outPath

    @staticmethod
    def data_separation(data: list, dividing_percentage: int) -> list:
        """
         Segmenting incoming data

        :param data: data to be processed
        :param dividing_percentage: percentage of data fragmented
        :return: predicted masks
        """
        percentage = int(len(data) / 100 * dividing_percentage)

        for i in range(0, len(data), percentage):
            # Create an index range for l of n items:
            yield data[i:i + percentage]

    @staticmethod
    def get_exif_information(img_info):
        """

        :param img_info: exif object
        :return: (lat, lon), orientation, (Height, Width), FocalLength, Altitude,
        """
        information = {}
        data = img_info.extract_exif()
        try:
            information["model"] = data["model"]
            information["coordx"] = data["gps"]["latitude"]
            information["coordy"] = data["gps"]["longitude"]
            information["width"] = data["width"]
            information["height"] = data["height"]

            # Focal Length
            fLen_obj = data["gps"]["FocalLength"]
            fLen_str = f"{fLen_obj}"
            fLen_arr = fLen_str.split("/")
            fLen = float(int(fLen_arr[0]) / int(fLen_arr[1]))
            information["FocalLength"] = fLen

            hor_width = data["height"] if data["orientation"] == 1 else data["width"]
            information["orientation"] = hor_width
            # Angle of View
            aFov = np.arctan(hor_width / (2 * fLen)) * (180 / np.pi)
            information["FoV"] = aFov
        except:
            raise Exception(f"Check the image Exif Data some missing values")

        for k, v in information.items():
            if information[k] is None:
                raise Exception(f"{k} is None")
            else:
                pass

        return information


"""
Mapilio Helper-tools is a python library for some image, prediction functions

Rule -> math functions
Exceptions -> handler errors or disable some warnings
Convertors -> tensorFaster_to_np, tensorMask_to_np, list_to_dict, url_to_image, Bunch
Generators ->path url creator, data separator, url to image
Logger -> time log, time save, file exist
"""


from .generator import Generator
from .logger import Logger, TimeLogger
from .handler import Handler
from .convertor import Convertor
from .rule import Utilities


from .util import __version__, get_version


from json import JSONEncoder

import cv2
import numpy as np
import urllib.request
from decimal import *


class Convertor:

    @staticmethod
    def url_to_image(url: str) -> np.ndarray:
        """
        # download the image, convert it to a NumPy array, and then read
        # it into OpenCV format
        :param url:
        :return:
        """

        resp = urllib.request.urlopen(url)
        image = np.asarray(bytearray(resp.read()), dtype="uint8")
        image = cv2.imdecode(image, cv2.IMREAD_COLOR)

        return image

    @staticmethod
    def tensorFaster_to_np(tensor):
        return tensor[0]["boxes"].cpu().numpy(), \
               tensor[0]["scores"].cpu().numpy(), \
               tensor[0]["labels"].cpu().numpy()

    @staticmethod
    def tensorMask_to_np(tensor):
        return tensor[0]["boxes"].detach().cpu().numpy(), \
               tensor[0]["scores"].detach().cpu().numpy(), \
               tensor[0]["labels"].detach().cpu().numpy(), \
               tensor[0]["masks"].squeeze().detach().cpu().numpy()

    @staticmethod
    def list_to_dict(data: list, config: dict) -> dict:
        """

        :param data:
        :param config:
        :return:
        """
        keys = config.columnKeys

        return dict(zip(keys, data))

    @staticmethod
    class Bunch(object):
        def __init__(self, adict):
            self.__dict__.update(adict)

    @staticmethod
    class numpy_array_encoder(JSONEncoder):
        def default(self, obj):
            if isinstance(obj, np.ndarray):
                return obj.tolist()
            return JSONEncoder.default(self, obj)

    @staticmethod
    def decimal_fix(number):
        getcontext().rounding = ROUND_DOWN
        return Decimal(number).quantize(Decimal(10) ** -9)

    @staticmethod
    def intersection_float_to_decimal(lat1, lon1, lat2, lon2):
        """

        Parameters
        ----------
        lat1
        lon1
        lat2
        lon2

        Returns points convert decimal format
        -------

        """
        lat1 = Convertor.decimal_fix(lat1)
        lon1 = Convertor.decimal_fix(lon1)
        lat2 = Convertor.decimal_fix(lat2)
        lon2 = Convertor.decimal_fix(lon2)

        return lat1, lon1, lat2, lon2